#include "demo.H"
#include "visitmodule.H"
//IMPORTANT NOTICE: All lines that run visit from inside this program must be set inside "if(myid==0){}" blocks, otherwise the viewer doesn't work right.

#include <iostream>
#include <cstdlib>
#include <string>
#include <fstream.h>
#include <cmath>
#include <ctime>

extern "C" void cli_initvisit(int, bool, int, char **);

bool quitFlag;
int commandFlag;

void ProcessCommand(string cmd);
void ControlCommandCallback(const char *cmd, int int_data, float float_data, const char *string_data);
Attribute *squareData(Attribute *data, string &newName, string &meshName);

int main(int argc, char *argv[]){
  //Initialize Basin (Step 1)
    int numprocs,myid,choice = -3;
    basin_initialize(myid,numprocs);
    berr << "-------VisIt Demo for BASIN Data-------"<<argc<<"\n\n";
    int debugLevel = 0;
    for(int i=0; i<argc; i++){
      if(strcmp(argv[i],"-debug")==0){
	debugLevel=1;
	if(i+1 < argc && isdigit(*(argv[i+1])))
	  debugLevel = atoi(argv[++i]);
	if(debugLevel < 0)
	  debugLevel = 0;
	if(debugLevel > 5)
	  debugLevel = 5;
      }
    }

    int plotTypeIndex = -1;
    if(myid==0){
      Py_Initialize();
      PyEval_InitThreads();
      Py_SetProgramName(argv[0]);
      PySys_SetArgv(argc,argv);
      cli_initvisit(debugLevel,false,argc,argv);

      PyRun_SimpleString((char*)"from visit import *");
      PyRun_SimpleString((char*)"Launch()");

      char *plotName = "Pseudocolor";
      PlotPluginManager *pluginManager = PlotPluginManager::Instance();
      //int plotTypeIndex = -1;
      for(int i = 0; i < pluginManager->GetNEnabledPlugins(); ++i){
	std::string id(pluginManager->GetEnabledID(i));
	ScriptingPlotPluginInfo *info=pluginManager->GetScriptingPluginInfo(id);
	if(info == 0)
	  continue;
	if(info->GetName() == std::string(plotName)){
	  plotTypeIndex=i;
	  break;
	}
      }
      if(plotTypeIndex <0){
	cout << "Error finding plot type. \n";
	return -1;
      }
    }

    MPI_Barrier(MPI_COMM_WORLD);

    //Reading files to BASIN Regions and copying data into grids and lists
    //(Step 2)
    Region cluster("cluster1.dat");
    List *l1 = cluster.get_list("RegionList");
    Region den("densfield.dat");
    Grid* g1 = den.get_grid();
    Geometry geom;

    vector<Attribute*> attrPtrs;
    vector<string> varname;
    //extracting Attributes from list
    Attribute *X= new Attribute(l1->get_attribute("x"));
    Attribute *Y= new Attribute(l1->get_attribute("y"));
    Attribute *Z= new Attribute(l1->get_attribute("z"));
    string meshname = "mesh";
    varname.push_back("mass");

    //exposing data for access to visit
    if(makePointMesh(meshname,X,Y,Z)!=0) //creates a Mesh
      berr <<"The mesh \""<<meshname<<"\" already exists. No mesh was created.\n";
    Attribute *data= new Attribute(l1->get_attribute("mass")); //creates a variable
    attrPtrs.push_back(data);
    if(makeVariable(varname[0],data,meshname)!=0)
      berr <<"The variable \""<<varname[0]<<"\" already exists. No variable was created.\n";
    vector<int> grid(3,40);
    geom = *(cluster.get_geometry());
    vector<float> ll = geom.get_bound(0);
    vector<float> ur = geom.get_bound(1);
    varname.push_back("mass_dens"); meshname = "griddedMesh";
    Attribute mass;
    Attribute *density =new Attribute(compute_density(*X,*Y,*Z,mass,grid,ll,ur,0));
    attrPtrs.push_back(density);
    //creating a grid using list data
    if(makeGridMesh(meshname,ll,ur,grid))
      berr <<"The mesh \""<<meshname<<"\" already exists. No mesh was created.\n";
    if( makeVariable(varname[1], density, meshname) )
      berr <<"The variable \""<<varname[1]<<"\" already exists. No variable was created.\n";
    Attribute *d1=new Attribute(g1->get_attribute("density"));
    geom=*(den.get_geometry());
    ll = geom.get_bound(0);
    ur = geom.get_bound(1);
    vector<int> nx = g1->get_nx();
    varname.push_back("density"); meshname = "grid";
    attrPtrs.push_back(d1);
    if(makeGridMesh(meshname,ll,ur,nx)) //creating grid from a grid file
      berr <<"The mesh \""<<meshname<<"\" already exists. No mesh was created.\n";
    if(makeVariable(varname[2],d1,meshname))//variable for above grid
      berr <<"The variable \""<<varname[2]<<"\" already exists. No variable was created.\n";

    //set up the environment (Step 3)
    SetupEnvironment();

    {
      //Create and process the list of exposed functions (Step 4)
      //IMPORTANT NOTE: The maximum number of commands that can
      //      be displayed on the simulation window is 6. Also,
      //      exposing 6 names thru the command setGenCMD prevents
      //      the Custom Command button from showing up in the
      //      simulation window.
      vector<string> names(5);
      names[0]="check";
      names[1]="halt";
      names[2]="square";
      names[3]="show";
      names[4]="addPts";
      setGenCMD(names);
    }

    if(myid==0){
      VisItInitializeSocketAndDumpSimFile("Demo","Dynamic Data simulation","/no/useful/path",NULL,"SimGUI_1.ui");
      cerr << "Known commands:\n";
      cerr << "\tplot   :\tplots data\n";
      cerr << "\tsquare :\tsquares newest variable\n";
      cerr << "\tcheck  :\tStops a run\n";
      cerr << "\tadd    :\tAdd new data points to all variables\n";
      cerr << "\tquit   :\texit code\n";
    }
    MPI_Bcast(simulationFileName,1024,MPI_CHAR,0,MPI_COMM_WORLD);
    char *fileloc="/home/nbeers/.visit/simulations/001158000461.Demo.sim1";
    if(myid==0)  viewer->OpenDatabase(fileloc/*simulationFileName*/); //Change this line to open the diamond data file vtext.okc. IE: plot something you know works in a questionable environment. Determine if visit and Basin are interfering with each other completely or only when Visit tries to plot Basin in this environment.
    //The above suggestion did not work.
    MPI_Barrier(MPI_COMM_WORLD);
    berr << "command> ";
    //Loop that contains code for connecting to VisIt (step 5)
    //IMPORTANT: The manner in which one breaks out of the loop can be triggered any way that you, the designer, see fit.
    //           This demo happens to make use of a global boolean flag "quitFlag" to do so. The manner in which the flag is triggered
    //           requires manual input from the user at the console. There are other methods for exiting this loop. (See below for 1 suggestion).
    bool drawn = false, draw=false, addPlot = false;
    while(!quitFlag){
        int visitstate;
	if(draw && false){
	  berr << "Plotting.\n";
	  int *d = new int;
	  *d = drawn;
	  if(myid==0 && choice>0 && choice <= varname.size()){
	    berr << "Checking drawn flag.\n";
	    if(!drawn){
	      berr << "Better be drawing.\n";
	      viewer->AddPlot(plotTypeIndex,varname[choice-1].c_str());
	      viewer->DrawPlots();
	      drawn=true;
	      *d = drawn;
	    }else
	      viewer->ChangeActivePlotsVar(varname[choice-1].c_str());
	  }else{
	    berr <<"Invalid selection. Please activate a variable first.\n";
	    berr <<"Use the 'check' command to do so.\n";
	  }
	  MPI_Bcast(d,1,MPI_INT,0,MPI_COMM_WORLD);
	  drawn = *d;
	  delete d;
	  MPI_Barrier(MPI_COMM_WORLD);
	  draw=false;
	}
	//cerr <<"ID "<<myid<<" is about to get call from something.\n";
	if(myid==0){
	  int blocking = (int) !runFlag;
	  visitstate = VisItDetectInput(blocking, fileno(stdin));
	}
	MPI_Bcast(&visitstate, 1, MPI_INT, 0, MPI_COMM_WORLD);
	cerr <<"ID "<<myid<<" received visitstate "<<visitstate<<endl;
	commandFlag = -1;
	switch(visitstate){
	  case -5:
	  case -4:
	  case -3:
	  case -2:
	  case -1:
	    cerr << "\nUnrecoverable error... exiting.\n";
	    quitFlag = true;
	    break;
	  case 0:
	    break;
	  case 1:
	    //The VisIt connection actually occurs here.
	    //runFlag = false;
	    if(!VisItAttemptToCompleteConnection()){
	      string err = VisItGetLastError();
	      if(err.length() > 0)
		cerr << err << endl;
	      else
		cerr << "VisIt failed to connect successfully\n";
	    }else{
	        cerr << "Connected Successfully!\n";
		VisItSetSlaveProcessCallback(SlaveProcessCallback);
		VisItSetCommandCallback(ControlCommandCallback);
	    }
	    //if(myid==0){
	      //viewer->AddPlot(plotTypeIndex,varname[0]);
	      //viewer->DrawPlots();
	    //}
	    //drawn=true;
	    break;
	  case 2:
	    //Depending on what is going on, you may only need to connect once at this point and want your code to continue on once VisIt has disconnected.
	    //If that is the case, a command to alter the quitFlag's value to true here would do just that.
	    berr << "Visit input detected. Processing VisIt Command. ";
	    if(!ProcessVisItCommand())
	      VisItDisconnect();
	    //else
	    //berr << "processing visit command\n";
	    berr << endl;
	    break;
	  case 3:
	    {
	      char buff[100];
	      string input;
	      if(myid==0){
		cin >> input;
		if(cin.eof()){
		  input = "quit";
		  cerr << "quit\n";
		}
	      }
	      if(input.length() > 0 && input[input.length()-1] == '\n')
		input[input.length()-1] = '\0';
	      if(numprocs>1){
		strcpy(buff,input.c_str());
		MPI_Bcast(buff,100,MPI_CHAR,0,MPI_COMM_WORLD);
		if(myid!=0)
		  input = string(buff);
	      }
	      //cerr << myid << ": input = " << input << endl;
	      ProcessCommand(input);
	    }
	    break;
	}
	if(!quitFlag && (commandFlag>=0)){
	  switch(commandFlag){
	  case 0:
	    time_t starttime = time(NULL);
	    berr << "Pause..";
	    while(time(NULL)==starttime){
	      //pause;
	    }
	    berr << "..GO!\n";
	    break;
	  case 1:
	    {
	      int *d = new int;
	      *d = drawn;
	      if(myid==0 && choice>0 && choice <= varname.size()){
		berr << "About to attempt plot.\n";
		if(!drawn){
		  berr << "never drawn before. Creating new plot.\n";
		  viewer->AddPlot(plotTypeIndex,varname[choice-1].c_str());
		  //viewer->DrawPlots();
		  drawn=true;
		  *d = drawn;
		}else{
		  berr << "Plot already created. Changing variable.\n";
		  viewer->ChangeActivePlotsVar(varname[choice-1].c_str());
		}
	      }else{
		berr <<"Invalid selection. Please activate a variable first.\n";
		berr <<"Use the 'check' command to do so.\n";
	      }
	      MPI_Bcast(d,1,MPI_INT,0,MPI_COMM_WORLD);
	      drawn = *d;
	      delete d;
	      MPI_Barrier(MPI_COMM_WORLD);
	      draw=true;
	      StopSimulation();
	      berr <<"  Data will be plotted.\n";
	    }
	    break;
	  case 2:
	    {
	      stringstream name;
	      name << "mass" << attrPtrs.size();
	      string temp1="mesh", temp2 = name.str();
	      attrPtrs.push_back(squareData(attrPtrs[0],temp2,temp1));
	      varname.push_back(temp2);
	      berr <<"  Data squared.\n";
	    }
	    StopSimulation();
	    break;
	  case 3:
	    //multiplyData();
// 	    {
// 	      int multiplier = 1;
// 	      berr <<"(NI)  Data multiplied by " << multiplier << ".\n";
// 	    }
	    if(choice>0 && choice <=varname.size()){
	      berr << "Length of array: " <<(attrPtrs[choice-1]->get_dims())[0];
	      int lo=0, hi = (attrPtrs[choice-1]->get_dims())[0];
	      double *temp = new double[hi];
	      attrPtrs[choice-1]->get(lo,hi-1,temp);
	      for(int i=0; i<hi; i+=10){
		berr << temp[i] << " ";
		if(i>0 && i%100 == 0) berr << endl;
	      }
	    }
	    berr << endl;
	    StopSimulation();
	    break;
	  case 4:
	    {
	      berr <<"Select one of the exposed variables as active.\n";
	      berr <<"Your choices are as follows.\n";
	      for(int i=1; i<=varname.size(); i++)
		berr << "\t"<<i<<") "<<varname[i-1]<<endl;
	      char trash[100]; choice= -3;
	      do{
		if(myid==0){
		  cerr << "The active variable will now be:";
		  cin >> trash;
		}
		MPI_Bcast(trash,100,MPI_CHAR,0,MPI_COMM_WORLD);
		MPI_Barrier(MPI_COMM_WORLD);
		choice = atoi(trash);
		if(choice<=0 || choice>varname.size())
		  berr <<"That is not a valid choice. Please choose again.\n";
	      }while(choice<=0 || choice > varname.size());
	      berr <<"  Check finished.\n";
	      StopSimulation();
	    }
	    break;
	  case 5:
	    //runFlag = false;
	    berr <<"(NI)  Data added here.\n";//addPoints();
	    {
	      if(drawn && !addPlot){
		berr << "Actually attempting to draw plot.";
		addPlot=!addPlot;
		if(myid==0) viewer->DrawPlots();
	      }else
		berr << "Already drew the plot.\n";
	    }
	    StopSimulation();
	    break;
	  case 6:
	    VisItDisconnect();
	    break;
	  case 7:
	    VisItAttemptToCompleteConnection();
	    break;
	  case 8:
	    VisItProcessEngineCommand();
	    break;
	  case 9:
	    
	    break;
	  default:
	    //cerr << cmd << " Is not a recognized command.\n";
	    berr << "  Custom command received. Processing...\n";
	    //if(CustomCommandParser(cmd))
	    //berr << cmd << "  is not a recognized command.\n";
	    berr << " Custom command processing not yet implemented.\n";
	  }
	}
	if(!quitFlag && !runFlag && visitstate==3){
	  berr << "command> ";
	}

    }
    cerr << "Good-bye\n";
    //Clean up BASIN and end main(). (Step 6)
    basin_finalize();
    return 0;
}


//Purpose: interprets commands input by the console or sent by VisIt.
//Each possible console input is assigned a number. When the commands are received from either the console or VisIt, they are sent thru ProcessCommand. That alters the variable commandFlag. Altering that variable allows the simulation above to access the function which corresponds to the selected commandFlag value. It also reduces the number of global variables required by the program.
void ProcessCommand(string cmd){
  //cerr << myid << " started processing the command.\n";
    if(cmd.compare("quit")==0 || cmd.compare("exit")==0)
      quitFlag = true;
    else if(cmd.compare("halt")==0)
      commandFlag = 0;
    else if(cmd.compare("plot")==0)
      commandFlag = 1;
    else if(cmd.compare("square")==0)
      commandFlag = 2;
    else if(cmd.compare("mult")==0 || cmd.compare("Multiply")==0||cmd.compare("show")==0)
      commandFlag = 3;
    else if(cmd.compare("check")==0 || cmd.compare("checkLast")==0)
      commandFlag = 4;
    else if(cmd.compare("add")==0 || cmd.compare("addPts")==0)
      commandFlag = 5;
    else if(cmd.compare("visit_disconnect")==0)
      commandFlag = 6;
    else if(cmd.compare("visit_connect")==0)
      commandFlag = 7;
    else if(cmd.compare("visit_process")==0)
      commandFlag = 8;
    else if(cmd.compare("")==0)
      commandFlag = 9;
    else
      commandFlag = 100;
}

//Create ControlCommandCallback(). (Step 7)
void ControlCommandCallback(const char *cmd, int int_data, float float_data, const char *string_data){
    berr << " ControlCommandCallback(cmd=" << cmd << ")\n";
    string buff = string(cmd);
    ProcessCommand(buff);
}

Attribute *squareData(Attribute *data, string &newName, string &meshName){
    Attribute *var = new Attribute(pow(*data + 0, 2.0));
    makeVariable(newName, data, meshName);
    return var;
}
