 /* Programmer: Jeremy Meredith
 * Date      : April  4, 2005
 *
 * Modifications:
 *    Jeremy Meredith, Thu Apr 28 18:17:02 PDT 2005
 *    Changed it to a rectilinear mesh.
 *
 *    Jeremy Meredith, Wed May 11 11:05:50 PDT 2005
 *    Added ghost zones.  Added domain lists for restricted load balancing.
 *
 *    Shelly Prevost added custom command updating
 */

#include "demoHconnection.H"
//#include "demoHvis.h"
#include "SimHelperFunc.H"

VisIt_SimulationMetaData *md = NULL; 
vector<Mesh> meshVect;
vector<demoVarB> visVarB;
int numdomains = 1;
bool runFlag = false;
int cycle = 0;

// Here is where you add all the initialization code you
// want to run only once.
void InitializeMD(int MaxNumCustCMD )
{
    md = (VisIt_SimulationMetaData*) malloc(sizeof(VisIt_SimulationMetaData));
    md->currentCycle = cycle;
    md->currentTime  = 0;
    md->currentMode  = runFlag ? VISIT_SIMMODE_RUNNING : VISIT_SIMMODE_STOPPED;
    
    md->numMeshes = meshVect.size();
    md->meshes = (VisIt_MeshMetaData*) malloc(sizeof(VisIt_MeshMetaData) * md->numMeshes);

    for(int i=0; i<md->numMeshes; i++){
      md->meshes[i].name = strdup(meshVect[i].name->c_str());
      md->meshes[i].meshType = meshVect[i].type;//VISIT_MESHTYPE_POINT;
      md->meshes[i].topologicalDimension = meshVect[i].nDims;
      md->meshes[i].spatialDimension = meshVect[i].nDims;
      md->meshes[i].numBlocks = numdomains;
      md->meshes[i].blockTitle = strdup("Domains");
      md->meshes[i].blockPieceName = strdup("domain");
      md->meshes[i].numGroups = 0;
      md->meshes[i].units = NULL;
      md->meshes[i].xLabel = NULL;
      md->meshes[i].yLabel = NULL;
      md->meshes[i].zLabel = NULL;
    }
    md->numScalars = visVarB.size();
    md->scalars = (VisIt_ScalarMetaData*) malloc(sizeof(VisIt_ScalarMetaData) * md->numScalars);

    for(int i=0; i<md->numScalars; i++){
      md->scalars[i].name = strdup(visVarB[i].name->c_str());
      md->scalars[i].meshName = strdup(visVarB[i].meshName);
      md->scalars[i].centering = VISIT_VARCENTERING_NODE;
      md->scalars[i].treatAsASCII = 0;
    }
    md->numMaterials   = 0;
    md->numCurves      = 0;
    md->numExpressions = 0;
    
    // this will set up the generic and custom commands
    initAllCMD(MaxNumCustCMD);
}                    

VisIt_SimulationMetaData *VisItGetMetaData()
{
  // maximum number of UI components connections
  // that you will be creating
int MAX_NUMBER_CUST_CMD = 15;

  // if the first time setup the meta data
  // and create meta data slots for the UI control
  // Do this only once.
  if (md == NULL )
  {
     InitializeMD(MAX_NUMBER_CUST_CMD);
     
     // set up the specific ui channel to control the
     // ui compenents in the VisIt custom commands dialog
     // These names should be the exact same name as defined
     // in the interface file.

     createCMD ( "MainTextLabel");
     createCMD ( "ShellySpinBox1");
     createCMD ( "progressBar1");
     createCMD ( "Top_Button_1");
     createCMD ( "ShellySlider_1");
     createCMD ( "ShellyDial_1");
     createCMD ( "ShellyLineEdit1");
     createCMD ( "ShellyText_2");
     createCMD ( "RadioButton1");
     createCMD ( "RadioButton2");
     createCMD ( "RadioButton3");
     createCMD ( "LCDNumber1");
     createCMD ( "CheckBox1");
     createCMD ( "timeEdit1");
     createCMD ( "dateEdit1");

   }

    // now update the meta data so that the custom
    // command interface will be brought up to date
    // with what is going on in the simulation
    Update_UI_Commands();

    // no return the updated meta data that will
    // be sent to the VisIt program
    return md;
}

// Here is where you want to put the changing information.
// This information will be sent back to VisIt and update
// the matching UI components.
void Update_UI_Commands()
{
      int MAX_CMD_STR_LEN = 64;
      // put updated UI information here.
      static int timeStep = 0;
      char value[MAX_CMD_STR_LEN];
      char modValue[MAX_CMD_STR_LEN];
#ifdef DEBUG_PRINT
      printf ( "updating UI command data \n");
#endif
     // move the progess bar and update the value in the spin box
      setCMDValue ("progressBar1",  (timeStep *10)% 100);
      setCMDValue ("ShellySpinBox1", timeStep);
      setCMDValue ("LCDNumber1",timeStep);

      // change the lable on the pushbutton
      setCMDText  ("Top_Button_1",  "Simulation Text");
      setCMDEnable ("Top_Button_1", 0);
      setCMDIsOn ("CheckBox1", 0);
      setCMDIsOn ("RadioButton1", 1);
      setCMDIsOn ("RadioButton2", 0);
                                          
      setCMDText ("RadioButton1", "Label 1");
      setCMDText ("RadioButton2", "Label 2");
      setCMDText ("RadioButton3", "Label 3");


      // set the text on the lable
      sprintf (  value, "%5d", timeStep *10);
      strcat( value, " Simulation Label");
      setCMDText  ("MainTextLabel", value);
      setCMDText  ("CheckBox1", "New Label");
      setCMDText  ("timeEdit1", "11:06:03");
      setCMDText  ("dateEdit1", "Mon Jan 23 2006");

      // move the slider and dial
      setCMDValue ( "ShellySlider_1",(timeStep * 10) % 100);
      setCMDValue ( "ShellyDial_1",timeStep % 360);

      sprintf (  modValue, "%5d", timeStep % 360);
      setCMDText ( "ShellyLineEdit1",modValue);
      setCMDText ( "ShellyText_2",modValue);

      md->currentCycle = cycle;
      md->currentTime  = timeStep;
      md->currentMode  = runFlag ? VISIT_SIMMODE_RUNNING : VISIT_SIMMODE_STOPPED;
      timeStep++;
}

// This function is called when Visit wants to retrieve
// the mesh data from the simulation

VisIt_MeshData *VisItGetMesh(int domain, const char *name){
    string cName(name);
    //if(BASIN_ID==0) cerr << "Grabbing a mesh.";
    bool found = false;
    int select;
    for(int i=0; i<md->numMeshes; i++){
      select = i;
      if(cName.compare(md->meshes[i].name)==0){
	found = true;
	break;
      }
    }
    if(!found){
      if(BASIN_ID==0) cerr << "Nothing was found?";
      return NULL;
    }else{
      switch(md->meshes[select].meshType){
      case VISIT_MESHTYPE_RECTILINEAR:
	{
	  VisIt_MeshData *mesh = new VisIt_MeshData;
	  
	  mesh->meshType = md->meshes[select].meshType;
	  mesh->rmesh = new VisIt_RectilinearMesh;
	  mesh->rmesh->ndims = meshVect[select].nDims;
	  mesh->rmesh->baseIndex[0] = 0;
	  mesh->rmesh->baseIndex[1] = 0;
	  mesh->rmesh->baseIndex[2] = 0;

	  mesh->rmesh->minRealIndex[0] = 0;
	  mesh->rmesh->minRealIndex[1] = 0;
	  mesh->rmesh->minRealIndex[2] = 0;

	  mesh->rmesh->maxRealIndex[0] = (meshVect[select].gm->nx)[0]-1;
	  mesh->rmesh->maxRealIndex[1] = (meshVect[select].gm->nx)[1]-1;
	  if(meshVect[select].nDims == 3)
	    mesh->rmesh->maxRealIndex[2] = (meshVect[select].gm->nx)[2]-1;
	  else
	    mesh->rmesh->maxRealIndex[2] = int(0);

	  mesh->rmesh->dims[0] = (meshVect[select].gm->nx)[0];
	  mesh->rmesh->dims[1] = (meshVect[select].gm->nx)[1];
	  if(meshVect[select].nDims == 3)
	    mesh->rmesh->dims[2] = (meshVect[select].gm->nx)[2];
	  else
	    mesh->rmesh->dims[2] = int(1);

	  double *x = new double[(meshVect[select].gm->nx)[0]];
	  double *y = new double[(meshVect[select].gm->nx)[1]];
	  double *z;
	  if(meshVect[select].nDims == 3)
	    z = new double[(meshVect[select].gm->nx)[2]];
	  else
	    z = new double(0);
	  double *min = meshVect[select].gm->min, *max = meshVect[select].gm->max;

	  for(int i=0; i< (meshVect[select].gm->nx)[0]; i++)
	    x[i] = (max[0]-min[0])*double(i)/(meshVect[select].gm->nx)[0] + min[0];
	  for(int i=0; i< (meshVect[select].gm->nx)[1]; i++)
	    y[i] = (max[1]-min[1])*double(i)/(meshVect[select].gm->nx)[1] + min[1];
	  if(meshVect[select].nDims == 3)
	    for(int i=0; i< (meshVect[select].gm->nx)[2]; i++)
	      z[i] = (max[2]-min[2])*double(i)/(meshVect[select].gm->nx)[2] + min[2];

	  mesh->rmesh->xcoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,x);
	  mesh->rmesh->ycoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,y);
	  mesh->rmesh->zcoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,z);

	  return mesh;
	  break;
	}
      case VISIT_MESHTYPE_POINT:
	{
	  VisIt_MeshData *mesh = new VisIt_MeshData;

	  mesh->meshType = md->meshes[select].meshType;
	  mesh->pmesh = new VisIt_PointMesh;
	  mesh->pmesh->ndims = meshVect[select].nDims;
	  mesh->pmesh->nnodes = meshVect[select].pm->numPoints;

	  double *x = new double[meshVect[select].pm->numPoints];
	  double *y = new double[meshVect[select].pm->numPoints];
	  double *z;
	  if(meshVect[select].nDims==3)
	    z = new double[meshVect[select].pm->numPoints];
	  else
	    z = new double(0);
	  (meshVect[select].pm->X)->get(0,meshVect[select].pm->numPoints -1,x);
	  (meshVect[select].pm->Y)->get(0,meshVect[select].pm->numPoints -1,y);
	  if(meshVect[select].nDims==3)
	    (meshVect[select].pm->Z)->get(0,meshVect[select].pm->numPoints -1,z);

	  mesh->pmesh->xcoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,x);
	  mesh->pmesh->ycoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,y);
	  mesh->pmesh->zcoords= VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,z);
	  
	  return mesh;
	  break;
	}
      default:
	{
	  if(BASIN_ID == 0) cerr << "asked for either an undefined mesh or a mesh of an unknown type.\nReturning NULL to VisIt.";
	  return NULL;
	}
      }
    }
    return NULL;
}


VisIt_MaterialData *VisItGetMaterial(int domain,const char *name)
{
    fprintf(stderr, "VisItGetMaterial\n");
    return NULL;
}

VisIt_ScalarData *VisItGetScalar(int domain,const char *name)
{
    string cName = string(name);
    //if(BASIN_ID==0) cerr << "Grabbing a Scalar.";
    bool found = false;
    int select;
    for(int i=0; i<md->numScalars; i++){
      select = i;
      if(cName.compare(*(visVarB[i].name))==0){
	found = true;
	break;
      }
    }
    if(!found){
      return NULL;
    }else{
      VisIt_ScalarData *sd = (VisIt_ScalarData*) malloc(sizeof(VisIt_ScalarData));
      sd->len = (visVarB[select].size);
      double *data = new double[ (visVarB[select].size) ];
      (visVarB[select].data)->get(0,visVarB[select].size-1,data);
      sd->data = VisIt_CreateDataArrayFromDouble(VISIT_OWNER_SIM,data);
      return sd;
    }
    return NULL;
}

VisIt_DomainList *VisItGetDomainList()
{
    int i;
    VisIt_DomainList *dl = (VisIt_DomainList*) malloc(sizeof(VisIt_DomainList));

    dl->nTotalDomains = numdomains;

    dl->nMyDomains = 1;
    dl->myDomains = VisIt_CreateDataArrayFromInt(VISIT_OWNER_VISIT,(int*) malloc(sizeof(int)));
    dl->myDomains.iArray[0] = BASIN_ID;

    return dl;
}

VisIt_SimulationCallback visitCallbacks =
{
    &VisItGetMetaData,
    &VisItGetMesh,
    NULL, /* material */
    NULL, /* species */
    &VisItGetScalar,
    NULL, /* curve */
    NULL,  /* mixed scalar */
    VisItGetDomainList
};

int makePointMesh(string &name,int nDims,Attribute *x, Attribute *y, Attribute *z){
    bool found = false;
    for(int i=0; i<meshVect.size(); i++){
      if(meshVect[i].name->compare(name)==0){
	found = true;
	break;
      }
    }
    if(!found){
      Mesh mTemp;
      pointMesh pTemp;
      
      mTemp.name = new string(name);
      mTemp.type = VISIT_MESHTYPE_POINT;
      mTemp.nDims= nDims;
      pTemp.X=x;
      pTemp.Y=y;
      if(nDims==3) pTemp.Z=z;
      pTemp.numPoints=(x->get_dims())[0];
      mTemp.pm = new pointMesh(pTemp);
      meshVect.push_back(mTemp);
      if(md!=NULL){
	md->numMeshes++;
	md->meshes = (VisIt_MeshMetaData*)realloc(md->meshes,sizeof(VisIt_MeshMetaData)*md->numMeshes);
	md->meshes[md->numMeshes-1].name = strdup(meshVect[md->numMeshes-1].name->c_str());
	md->meshes[md->numMeshes-1].meshType = meshVect[md->numMeshes-1].type;//VISIT_MESHTYPE_POINT;
	md->meshes[md->numMeshes-1].topologicalDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].spatialDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].numBlocks = numdomains;
	md->meshes[md->numMeshes-1].blockTitle = strdup("Domains");
	md->meshes[md->numMeshes-1].blockPieceName = strdup("domain");
	md->meshes[md->numMeshes-1].numGroups = 0;
	md->meshes[md->numMeshes-1].units = NULL;
	md->meshes[md->numMeshes-1].xLabel = NULL;
	md->meshes[md->numMeshes-1].yLabel = NULL;
	md->meshes[md->numMeshes-1].zLabel = NULL;
      }
      return 0;
    }else{
      return 1;
    }
}

int makePointMesh(string &name,int nDims,Attribute &x, Attribute &y, Attribute &z){
    bool found = false;
    for(int i=0; i<meshVect.size(); i++){
      if(meshVect[i].name->compare(name)==0){
	found = true;
	break;
      }
    }
    if(!found){
      Mesh mTemp;
      pointMesh pTemp;
      
      mTemp.name = new string(name);
      mTemp.type = VISIT_MESHTYPE_POINT;
      mTemp.nDims= nDims;
      pTemp.X=new Attribute(x);
      pTemp.Y=new Attribute(y);
      if(nDims==3) pTemp.Z=new Attribute(z);
      pTemp.numPoints=(x.get_dims())[0];
      mTemp.pm = new pointMesh(pTemp);
      meshVect.push_back(mTemp);
      if(md!=NULL){
	md->numMeshes++;
	md->meshes = (VisIt_MeshMetaData*)realloc(md->meshes,sizeof(VisIt_MeshMetaData)*md->numMeshes);
	md->meshes[md->numMeshes-1].name = strdup(meshVect[md->numMeshes-1].name->c_str());
	md->meshes[md->numMeshes-1].meshType = meshVect[md->numMeshes-1].type;//VISIT_MESHTYPE_POINT;
	md->meshes[md->numMeshes-1].topologicalDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].spatialDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].numBlocks = numdomains;
	md->meshes[md->numMeshes-1].blockTitle = strdup("Domains");
	md->meshes[md->numMeshes-1].blockPieceName = strdup("domain");
	md->meshes[md->numMeshes-1].numGroups = 0;
	md->meshes[md->numMeshes-1].units = NULL;
	md->meshes[md->numMeshes-1].xLabel = NULL;
	md->meshes[md->numMeshes-1].yLabel = NULL;
	md->meshes[md->numMeshes-1].zLabel = NULL;
      }
      return 0;
    }else{
      return 1;
    }
}

int makePointMesh(string &name, Attribute &x, Attribute &y, Attribute &z){
    return makePointMesh(name,3,x,y,z);
}

int makePointMesh(string &name, Attribute &x, Attribute &y){
    Attribute z;
    return makePointMesh(name,2,x,y,z);
}

int makePointMesh(string &name, Attribute *x, Attribute *y, Attribute *z){
    return makePointMesh(name,3,x,y,z);
}

int makePointMesh(string &name, Attribute *x, Attribute *y){
    Attribute *z;
    return makePointMesh(name,2,x,y,z);
}

int makeGridMesh(string &name,vector<float> &ll,vector<float> &ur,vector<int> &nx){
    bool found = false;
    for(int i=0; i<meshVect.size(); i++){
      if(meshVect[i].name->compare(name)==0){
	found = true;
	break;
      }
    }
    if(!found){
      Mesh mTemp;
      gridMesh gTemp;
      
      mTemp.name = new string(name);
      mTemp.type = VISIT_MESHTYPE_RECTILINEAR;
      mTemp.nDims = ll.size();
      gTemp.min = new double[mTemp.nDims];
      gTemp.max = new double[mTemp.nDims];
      gTemp.nx  = new int[mTemp.nDims];
      for(int i=0; i<mTemp.nDims; i++){
	gTemp.min[i]=double(ll[i]);
	gTemp.max[i]=double(ur[i]);
	gTemp.nx[i]=nx[i];
      }
      mTemp.gm = new gridMesh(gTemp);
      meshVect.push_back(mTemp);
      if(md!=NULL){
	md->numMeshes++;
	md->meshes = (VisIt_MeshMetaData*)realloc(md->meshes,sizeof(VisIt_MeshMetaData)*md->numMeshes);
	md->meshes[md->numMeshes-1].name = strdup(meshVect[md->numMeshes-1].name->c_str());
	md->meshes[md->numMeshes-1].meshType = meshVect[md->numMeshes-1].type;//VISIT_MESHTYPE_POINT;
	md->meshes[md->numMeshes-1].topologicalDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].spatialDimension = meshVect[md->numMeshes-1].nDims;
	md->meshes[md->numMeshes-1].numBlocks = numdomains;
	md->meshes[md->numMeshes-1].blockTitle = strdup("Domains");
	md->meshes[md->numMeshes-1].blockPieceName = strdup("domain");
	md->meshes[md->numMeshes-1].numGroups = 0;
	md->meshes[md->numMeshes-1].units = NULL;
	md->meshes[md->numMeshes-1].xLabel = NULL;
	md->meshes[md->numMeshes-1].yLabel = NULL;
	md->meshes[md->numMeshes-1].zLabel = NULL;
      }
      return 0;
    }else{
      return 1;
    }
}

int makeVariable(string &name, Attribute &data, string &meshName){
    if(meshVect.size()<1){
      return 2;
    }else{
      bool found = false;
      int mesh=0;
      for(int i=0; i<meshVect.size(); i++){
	mesh = i;
	if(meshVect[i].name->compare(meshName)==0){
	  int meshSize=1;
	  switch(meshVect[i].type){
	    case VISIT_MESHTYPE_RECTILINEAR:
	      meshSize*=(meshVect[i].gm->nx)[0];
	      meshSize*=(meshVect[i].gm->nx)[1];
	      if(meshVect[i].nDims == 3) meshSize*=(meshVect[i].gm->nx)[2];
	      int attSize;
	      if( (data.get_dims()).size()>1 ){
		attSize = 1;
		for(int i=0; i< (data.get_dims()).size(); i++)
		  attSize*=(data.get_dims())[i];
	      }else
		attSize = data.get_dims()[0];
	      if(attSize==meshSize)
		found = true;
	      else
		found = false;
	      break;
	    case VISIT_MESHTYPE_POINT:
	      if((meshVect[i].pm->numPoints) == data.get_dims()[0])
		found = true;
	      else
		found = false;
	      break;
	    default:
	      found = false;
	  }
	  break;
	}
      }
      if(!found){
	return 2;
      }else{
	found = false;
	for(int i=0; i<visVarB.size(); i++){
	  if(visVarB[i].name->compare(name)==0){
	    found = true;
	    break;
	  }
	}
	if(!found){
	  demoVarB vTemp;
	  vTemp.name = new string(name);
	  vTemp.meshName = strdup(meshName.c_str());
	  vTemp.data = new Attribute(data);
	  if(meshVect[mesh].type == VISIT_MESHTYPE_POINT){
	    vTemp.size = data.get_dims()[0];
	  }else if(meshVect[mesh].type == VISIT_MESHTYPE_RECTILINEAR){
	    vTemp.size = data.get_dims()[0]*data.get_dims()[1];
	    if(meshVect[mesh].nDims==3) vTemp.size*=data.get_dims()[2];
	  }else{
	    return 2;
	  }
	  //cerr << vTemp.size << "=?=" << (data.get_dims())[0]<<endl;
	  visVarB.push_back(vTemp);
	  if(md!=NULL){
	    md->numScalars++;
	    md->scalars = (VisIt_ScalarMetaData*)realloc(md->scalars, sizeof(VisIt_ScalarMetaData)*md->numScalars);
	    md->scalars[md->numScalars-1].name = strdup(visVarB[md->numScalars-1].name->c_str());
	    md->scalars[md->numScalars-1].meshName = strdup(visVarB[md->numScalars-1].meshName);
	    md->scalars[md->numScalars-1].centering = VISIT_VARCENTERING_NODE;
	    md->scalars[md->numScalars-1].treatAsASCII = 0;
	  }
	  return 0;
	}else{
	  return 1;
	}
      }
    }
}

int makeVariable(string &name, Attribute *data, string &meshName){
    if(meshVect.size()<1){
      return 2;
    }else{
      bool found = false;
      int mesh=0;
      for(int i=0; i<meshVect.size(); i++){
	mesh = i;
	if(meshVect[i].name->compare(meshName)==0){
	  int meshSize=1;
	  switch(meshVect[i].type){
	    case VISIT_MESHTYPE_RECTILINEAR:
	      meshSize*=(meshVect[i].gm->nx)[0];
	      meshSize*=(meshVect[i].gm->nx)[1];
	      if(meshVect[i].nDims == 3) meshSize*=(meshVect[i].gm->nx)[2];
	      int attSize;
	      if( (data->get_dims()).size()>1 ){
		attSize = 1;
		for(int i=0; i< (data->get_dims()).size(); i++)
		  attSize*=(data->get_dims())[i];
	      }else
		attSize = data->get_dims()[0];
	      if(attSize==meshSize)
		found = true;
	      else
		found = false;
	      break;
	    case VISIT_MESHTYPE_POINT:
	      if((meshVect[i].pm->numPoints) == data->get_dims()[0])
		found = true;
	      else
		found = false;
	      break;
	    default:
	      found = false;
	  }
	  break;
	}
      }
      if(!found){
	return 2;
      }else{
	found = false;
	for(int i=0; i<visVarB.size(); i++){
	  if(visVarB[i].name->compare(name)==0){
	    found = true;
	    break;
	  }
	}
	if(!found){
	  demoVarB vTemp;
	  vTemp.name = new string(name);
	  vTemp.meshName = strdup(meshName.c_str());
	  vTemp.data = data;
	  if(meshVect[mesh].type == VISIT_MESHTYPE_POINT){
	    vTemp.size = data->get_dims()[0];
	  }else if(meshVect[mesh].type == VISIT_MESHTYPE_RECTILINEAR){
	    vTemp.size = data->get_dims()[0]*data->get_dims()[1];
	    if(meshVect[mesh].nDims==3) vTemp.size*=data->get_dims()[2];
	  }else{
	    return 2;
	  }
	  //cerr << vTemp.size << "=?=" << (data.get_dims())[0]<<endl;
	  visVarB.push_back(vTemp);
	  if(md!=NULL){
	    md->numScalars++;
	    md->scalars = (VisIt_ScalarMetaData*)realloc(md->scalars, sizeof(VisIt_ScalarMetaData)*md->numScalars);
	    md->scalars[md->numScalars-1].name = strdup(visVarB[md->numScalars-1].name->c_str());
	    md->scalars[md->numScalars-1].meshName = strdup(visVarB[md->numScalars-1].meshName);
	    md->scalars[md->numScalars-1].centering = VISIT_VARCENTERING_NODE;
	    md->scalars[md->numScalars-1].treatAsASCII = 0;
	  }
	  return 0;
	}else{
	  return 1;
	}
      }
    }
}
